//
//  JMCyclePhotoView.m
//  JMCyclePhotoView
//
//  Created by DJM on 2018/3/16.
//  Copyright © 2018年 DJM. All rights reserved.
//

#import "JMCyclePhotoView.h"
#import "UIView+JMCyclePhotoViewExt.h"
#import "UIImageView+JMCyclePhotoViewExt.h"

@interface JMCyclePhotoView () <UIScrollViewDelegate>
/** 传入的原始数据 */
@property (nonatomic, copy) NSArray *dataArr;
/** 如果原始数据是字典数组，取值key */
@property (nonatomic, copy) NSString *key;
/** 图片点击回调 */
@property (nonatomic, copy) void (^MyBlock)(id data, NSInteger index);
//
/***/
@property (nonatomic, weak) UIScrollView *scr;
/** 左边的图片 */
@property (nonatomic, weak) UIImageView *leftImg;
/** 中间的图片 */
@property (nonatomic, weak) UIImageView *centerImg;
/** 右边的图片 */
@property (nonatomic, weak) UIImageView *rightImg;
/** 当前展示的序号 */
@property (nonatomic) NSInteger index;
/***/
@property (nonatomic, weak) UIPageControl *pageControl;
/***/
@property (nonatomic , strong) NSTimer *timer;
/***/
@property (nonatomic) BOOL isFirst;
/** 类型 0不处理 1地址 2image */
@property (nonatomic) NSInteger type;
/***/

@end

@implementation JMCyclePhotoView

/**
 初始化方法
 @param frame    轮播图的frame
 @param images   展示的图片的数组，支持image、string
 @param imageKey 如果数组中是一个字典，则取图的key，如果不是字典，则传nil
 @return 初始化
 */
- (instancetype)initWithFrame:(CGRect)frame images:(NSArray *)images imageKey:(NSString *)imageKey {
    if (self = [super initWithFrame:frame]) {
        self.dataArr = images;
        self.key = imageKey;
        [self dataTypeHandle];
        [self initUI];
    }
    return self;
}

/**
 图片点击回调
 @param callback 点击回调，返回当前点击的图的序号，以及当前的数据
 */
- (void)imageTapCallback:(void (^)(id, NSInteger))callback {
    _MyBlock = callback;
}



#pragma mark - action and click

- (void)imageTap:(UITapGestureRecognizer *)tap {
    if (_MyBlock) {
        self.MyBlock(self.dataArr[self.index], self.index);
    }
}



#pragma mark - UIScrollViewDelegate

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView {
    [self removeTimer];
}

- (void)scrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate {
    [self addTimer:self.timeMargin];
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView {
    [self scrollViewScrollHandle:scrollView];
}

- (void)scrollViewDidEndScrollingAnimation:(UIScrollView *)scrollView {
    [self scrollViewScrollHandle:scrollView];
}

- (void)scrollViewScrollHandle:(UIScrollView *)scrollView {
    
    NSInteger i = scrollView.contentOffset.x / scrollView.width - 1;
    NSInteger count = self.dataArr.count;
    
    if (i < 0) {
        self.index -= 1;
        if (self.index < 0) {
            self.index = count - 1;
        }
        self.rightImg.image = self.centerImg.image;
        self.centerImg.image = self.leftImg.image;
        NSInteger leftIndex = self.index - 1;
        if (leftIndex < 0) {
            leftIndex = count - 1;
        }
        
        id value = self.key ? self.dataArr[leftIndex][self.key] : self.dataArr[leftIndex];
        if (self.type == 1) {
            [self.leftImg downImageWithUrlString:value placeholder:nil];
        } else if (self.type == 2) {
            self.leftImg.image = value;
        }
    } else if (i > 0) {
        self.index += 1;
        if (self.index >= count) {
            self.index = 0;
        }
        self.leftImg.image = self.centerImg.image;
        self.centerImg.image = self.rightImg.image;
        NSInteger rightIndex = self.index + 1;
        if (rightIndex >= count) {
            rightIndex = 0;
        }
        
        id value = self.key ? self.dataArr[rightIndex][self.key] : self.dataArr[rightIndex];
        if (self.type == 1) {
            [self.rightImg downImageWithUrlString:value placeholder:nil];
        } else if (self.type == 2) {
            self.rightImg.image = value;
        }
    }
    
    scrollView.contentOffset = CGPointMake(scrollView.width, 0);
    self.pageControl.currentPage = self.index;
}



#pragma mark - Method

- (void)addTimer:(NSInteger)timer {
    self.timer = [NSTimer scheduledTimerWithTimeInterval:timer repeats:YES block:^(NSTimer * _Nonnull timer) {
        [self.scr setContentOffset:CGPointMake(self.scr.width * 2, 0) animated:YES];
    }];
}

- (void)removeTimer {
    if (_timer) {
        [_timer invalidate];
        _timer = nil;
    }
}

- (void)updateSubviewsWithFrame:(CGRect)frame {
 
    self.frame = frame;
    self.scr.frame = self.bounds;
    CGFloat w = frame.size.width, h = frame.size.height;
    self.leftImg.frame = CGRectMake(0, 0, w, h);
    self.centerImg.frame = CGRectMake(w, 0, w, h);
    self.rightImg.frame = CGRectMake(w * 2, 0, w, h);
    self.scr.contentSize = CGSizeMake(w * 3, 0);
    self.scr.contentOffset = CGPointMake(w, 0);
    CGFloat pageH = 10;
    self.pageControl.frame = CGRectMake(0, h - pageH, w, pageH);
}

- (void)willMoveToSuperview:(UIView *)newSuperview {
    if (!newSuperview) {
        [self removeTimer];
    }
}

- (void)dataTypeHandle {
    
    NSInteger stringCount = 0, imageCount = 0, count = self.dataArr.count;
    for (id d in self.dataArr) {
        id value = self.key ? d[self.key] : d;
        if ([value isKindOfClass:[NSString class]]) {
            stringCount += 1;
        } else if ([value isKindOfClass:[UIImage class]]) {
            imageCount += 1;
        }
    }
    if (stringCount == count) {
        self.type = 1;
    } else if (imageCount == count) {
        self.type = 2;
    }
}



#pragma mark - init UI

- (void)initUI {
 
    if (!self.type) {
        return;
    }
    
    UIScrollView *scr = [UIScrollView new];
    scr.bounces = NO;
    scr.delegate = self;
    scr.pagingEnabled = YES;
    scr.showsVerticalScrollIndicator = NO;
    scr.showsHorizontalScrollIndicator = NO;
    [self addSubview:scr];
    self.scr = scr;

    for (NSInteger i = 0; i < 3; i++) {
        UIImageView *img = [UIImageView new];
        img.userInteractionEnabled = YES;
        [img addGestureRecognizer:[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(imageTap:)]];
        img.backgroundColor = [UIColor colorWithRed:arc4random_uniform(255) / 255.0
                                              green:arc4random_uniform(255) / 255.0
                                               blue:arc4random_uniform(255) / 255.0
                                              alpha:1];
        [scr addSubview:img];
        
        if (self.type) {
            NSInteger index = i, count = self.dataArr.count;
            if (i >= count) {
                i = count - 1;
            }
            if (self.type == 1) {
                [img downImageWithUrlString:self.dataArr[index] placeholder:nil];
            } else if (self.type == 2) {
                img.image = self.dataArr[index];
            }
        }
        
        if (!i) {
            self.leftImg = img;
        } else if (i == 1) {
            self.centerImg = img;
        } else {
            self.rightImg = img;
        }
    }
    
    UIPageControl *pageControl = [UIPageControl new];
    pageControl.numberOfPages = self.dataArr.count;
    pageControl.pageIndicatorTintColor = [UIColor grayColor];
    pageControl.currentPageIndicatorTintColor = [UIColor whiteColor];
    [self addSubview:pageControl];
    self.pageControl = pageControl;
    
    [self addTimer:4];
    
    [self updateSubviewsWithFrame:self.frame];
}

/** 自动滚动的时间间隔，默认为4秒 */
- (void)setTimeMargin:(NSInteger)timeMargin {
    _timeMargin = timeMargin;
    [self removeTimer];
    [self addTimer:timeMargin];
}

@end
